/**
 * External dependencies
 */
import React, { useState, useEffect, useMemo } from 'react';
import axios from 'axios';

/**
 * Internal dependencies
 */
import { AppWrapper } from './wrappers';
import { ResultsLoader } from './loaders';
import {
	Listing,
	LoadMore,
	ErrorMessage,
	NoListings,
	NoResults,
	Filters,
	OrderBy,
} from './components';
import { ListingsWrapper } from './wrappers';
import { BaseFilterer, ListingsFilterer, URLParamsHandler } from './utils';
import { AppContext } from './utils/Context';

/**
 * Configuration
 */
import config from './config';
let { texts, actions, restBase, apiVersion, colors, searchOptions } = config;

const App = ({ listingsType, showOrderBy, showFilters, showRangeSliders, listingsPerPage, showLoadMore, admin }) => {
	const [listings, setListings] = useState([]);
	const [isLoading, setIsLoading] = useState(true);
	const [errorStatus, setErrorStatus] = useState(false);
	const [paging, setPaging] = useState(1);
	const [frontEndFilters, setFrontEndFilters] = useState({});

	// Remap values
	const windowConfig = window.blockLinearListingsConfig;
	texts = windowConfig.texts;
	actions = windowConfig.actions;
	restBase = windowConfig.restBase;
	apiVersion = windowConfig.apiVersion;
	colors = windowConfig.colors;
	searchOptions = windowConfig.searchOptions;

	// upon certain filters change
	useEffect(() => {

		// Populate URL params
		const onLoadParams = URLParamsHandler();
		if( Object.keys(onLoadParams).length ){
			setFrontEndFilters({ ...frontEndFilters, ...onLoadParams });
		}

		// Fetch data
		const action = actions.listings.replace('%s', apiVersion);

		// Load routes data
		axios
			.get(restBase + '/' + action, {
				params: {},
			})
			.then(function (response) {

				if( typeof response.data.errors !== 'undefined' ){
					setErrorStatus(true);
					setIsLoading(false);
					setListings([]);
				} else {
					setListings(response.data);
					setIsLoading(false);
				}

			})
			.catch(function (thrown) {

				setErrorStatus(true);
				setIsLoading(false);
				setListings([]);
				
			});
	}, []);

	// Pagination and filters reset
	useEffect(() => {
		setPaging(1);
	}, [listingsType, showFilters]);

	// Pagination loadmore
	const loadMore = () => {
		setPaging(paging + 1);
	};

	if( listings === {} ){
		setErrorStatus(true);
	}

	// Filter data by admin settings
	const baseListings = BaseFilterer({ listings, listingsType });

	// Filter by frontend filters
	let filteredListings = ListingsFilterer({
		listings: baseListings,
		filters: frontEndFilters,
		listingsType: listingsType,
	});

	// order elements
	if( typeof frontEndFilters['orderBy'] !== 'undefined' && frontEndFilters['orderBy'] === 'reverse'){
		filteredListings = filteredListings.reverse();
	}

	const context = useMemo(() => {
		return {
			texts,
			colors,
			listingsType,
			frontEndFilters,
			searchOptions,
			setFrontEndFilters
		};
	}, [listingsType, frontEndFilters]);

	return (
		<AppContext.Provider value={context}>
			<AppWrapper windowConfig={windowConfig}>
				{isLoading || errorStatus ? (
					<>
						{isLoading ? <ResultsLoader /> : ''}
						{errorStatus ? (
							<ErrorMessage />
						) : (
							''
						)}
					</>
				) : (
					<>
						<div className="linear-listings__handlers">
							<Filters showFilters={showFilters} showRangeSliders={showRangeSliders} />
							{ showOrderBy && (
								<OrderBy />
							)}
						</div>
						<ListingsWrapper windowConfig={windowConfig} listingsCount={filteredListings.length}>
							{filteredListings.length > 0 ? (
								<>
									{filteredListings
										.slice(0, paging * listingsPerPage)
										.map((listing, index) => {
											return (
												<Listing
													listing={listing}
													key={listing.id + index}
													listingsPerPage={
														listingsPerPage
													}
													index={index}
													admin={admin}
												/>
											);
										})}
								</>
							) : (
								<>
									{( ( Object.keys(frontEndFilters).length === 0 ) ? 
										<NoListings />
									:
										<NoResults />
									)}
								</>
							)}
						</ListingsWrapper>
						{showLoadMore &&
						filteredListings.length > paging * listingsPerPage ? (
							<LoadMore trigger={loadMore} />
						) : (
							''
						)}
					</>
				)}
			</AppWrapper>
		</AppContext.Provider>
	);
};

export default App;
