<?php
/**
 * Fetching content from api functionality
 *
 * @package Linear
 */

/**
 * Provides methods for fetching content from API.
 */
class Linear_Listing {

	/**
	 * Core plugin class instance.
	 *
	 * @var Linear
	 */
	protected static $linear;

	/**
	 * URL formats for listing links.
	 * %1$s is reserved for ID
	 *
	 * @var string
	 */
	protected static $formats = array(
		'dixu_bidding_url'  => 'https://dixu.fi/fi/tarjouskauppa/%2$s/%1$s',
		'dixu_listing_url'  => 'https://dixu.fi/fi/myytavat-asunnot/asunto/%2$s/%1$s',
		'listing_image_url_relative' => 'https://images.linear.fi/%1$s',
		'listing_image_url' => '%1$s',
	);
	
	/**
	 * Listing data.
	 *
	 * @var array
	 */
	public $data;

	/**
	 * Defines class properties.
	 */
	function __construct( $listing_data ) {
		
		$this->data = $listing_data;

		if ( is_null( self::$linear ) ) {
			self::$linear = Linear::get_instance();
		}

		if ( ! isset( self::$formats['single_listing_url'] ) ) {
			global $post;
			$single_listing_url = get_the_permalink( self::$linear->get_option( 'listings_page' ) );
			if ( $post->ID == self::$linear->get_option( 'rentals_page' ) ) {
				$single_listing_url = get_the_permalink( self::$linear->get_option( 'rentals_page' ) );
			} elseif ( $post->ID == self::$linear->get_option( 'workplace_page' ) ) {
				$single_listing_url = get_the_permalink( self::$linear->get_option( 'workplace_page' ) );
			}
			
			self::$formats['single_listing_url'] = str_replace( '%', '%%', $single_listing_url );
			if ( array_key_exists( 'query', parse_url( $single_listing_url ) ) ) {
				$single_listing_url = $single_listing_url . '&linear_listing_id=%1$s';
			} else {
				$single_listing_url = $single_listing_url . '%2$s/%1$s';
			}
			self::$formats['single_listing_url'] = $single_listing_url;
		}
	}

	/**
	 * Returns formated link for listing.
	 * 
	 * @param  string $format Specific link format.
	 * @return string
	 */
	public function get_listing_link( $format = 'single_listing_url' ) {
		return sprintf( self::$formats[$format], $this->data['id'], sanitize_title( $this->data['address'] ) );
	}

	/**
	 * Display formated and escaped link url for listing.
	 * 
	 * @param string $format Specific link format.
	 */
	public function the_listing_link( $format = 'single_listing_url' ) {
		echo esc_url( $this->get_listing_link( $format ) );
	}

	/**
	 * Returns image url for specific image id.
	 * 
	 * @param string|array $image Id for specific image.
	 * @return string
	 */
	public function get_image_url( $image ) {
		if ( !is_null($image) && strpos($image, 'https://images.linear.fi') !== false ){
			return sprintf( self::$formats['listing_image_url'], $image );
		}
		return sprintf( self::$formats['listing_image_url_relative'], $image );
	}

	/**
	 * Display escaped image url for by image id.
	 * 
	 * @param  string|array $image Id for specific image.
	 * @return string
	 */
	public function the_image_url( $image ) {
		echo esc_url( $this->get_image_url( $image ) );
	}

	/**
	 * Check if listing has data field, or if data equals to value.
	 * 
	 * @param  string $key    Key for specific data field.
	 * @param  mixed  $equals Value to compare if exists.
	 * @return boolean
	 */
	public function has_data( $key, $equals = null ) {
		if ( array_key_exists( $key, $this->data ) ) {
			$data = $this->data[$key];
			if ( ! ( is_null( $data ) || $data === '' || ( is_array( $this->data[$key] ) && count( $data ) === 0 ) ) ) {
				if ( ! is_null( $equals ) && $equals !== $data ) {
					return false;
				}
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Returns listing data by key.
	 * If data doesn't exist it will return $default value
	 * 
	 * @param  string $key     Key for specific data field.
	 * @param  mixed  $default Optional value.
	 * @return mixed
	 */
	public function get_data( $key, $default = null){
		if ( ! $this->has_data( $key ) ) {
			return $default;
		}
		return $this->data[$key];
	}

	/**
	 * Display listing data by key.
	 * If data doesn't exist it will display $default value
	 * 
	 * @param  string $key     Key for specific data field.
	 * @param  mixed  $default Optional value.
	 */
	public function the_data( $key, $default = null ){
		$data = $this->get_data( $key, $default );

		if ( is_array( $data ) ){
			// Remove null & falsy values from array... null can be in a array...
			$data = array_filter( $data );
			if ( $data === array_filter( $data, 'is_string' ) ) {
				$nonconstant_data = array_filter( $data, function( $item ) { return !ctype_upper( str_replace( '_', '', $item ) ); } );

				// Filter and translate constants
				$constant_data = array_filter( $data, function( $item ) { return ctype_upper( str_replace( '_', '', $item ) ); } );
				$constant_data = array_map(
					function ( $slug ) use ( $key ) {
						return self::$linear->get_slug_label( "$slug@$key" );
					},
					$constant_data
				);

				// Merge constants and non-constants
				$data = array_merge($constant_data, $nonconstant_data);

				// Pretty print data
				$data = wp_sprintf_l( '%l', $data );
			}
		} elseif ( is_string( $data ) ) {
			if ( $key === 'release' ) {
				if( $data === 'DATE'){
					if( $this->get_data('releaseDate')){
						$data = date_i18n( get_option( 'date_format' ), strtotime(  $this->get_data('releaseDate') ) );
					} else {
						$data = null;
					}
				}
			}
			if ( is_numeric( str_replace( ',', '.', $data ) ) ) {
				if ( ! preg_match( '/Year$/', $key ) ) {
					$data = self::$linear->number_format( str_replace( ',', '.', $data ) );
					// If key string ends with substring add suffixe
					if ( $key === 'area' || preg_match( '/Area$/', $key ) ) {
						$data =  $data . '&nbsp;m²';
					} elseif ( $key === 'propertyTax' || $key === 'landYearRent' ) {
						$data = $this->get_price_data( $key, self::$linear->get_slug_label( 'PER_YEAR' ) );
					} elseif ( $key === 'debt' || preg_match( '/(?:Price|Tax)$/', $key ) ){
						$data = $this->get_price_data( $key );
					} elseif ( $key === 'rent' || preg_match( '/(?:Charge|Charges|Rent)$/', $key ) ) {
						$data = $this->get_price_data( $key, $this->get_data_label( $key . 'Type', 'PER_MONTH' ) );
					} elseif ( $key === 'securityDeposit' ) {
						if ( $this->has_data( 'securityDepositType', 'CUSTOM' ) ) {
							$data = $this->get_data_label( 'customSecurityDeposit' );
						} elseif ( $this->has_data( 'securityDepositType', 'SINGLE_PAYMENT' ) ) {
							$data = $this->get_price_data( $key );
						} elseif ( $this->has_data( 'securityDepositType', 'EQUAL_TO_MONTHS_OF_RENT' ) ) {
							$data = $data . ' ' . $this->get_data_label( 'securityDepositType' );
						} else {
							$data = $this->get_price_data( $key, $this->get_data_label( 'securityDepositType' ) );
						}
					} elseif ( $key === 'electricHeatingPowerUsage' ) {
						$data = $data . ' ' . esc_html__(  'kWh / year', 'linear' );
					}
				}
			} elseif ( preg_match( '/(?:Date|At)$/', $key ) && strtotime( $data ) ) {
				$data = date_i18n( get_option( 'date_format' ), strtotime( $data ) );
			} elseif ( ctype_upper( str_replace( '_', '', $data ) ) ) {
				$data = self::$linear->get_slug_label( "$data@$key"  );
			} else {
				$data = nl2br( $data );
			}
		} elseif ( is_bool( $data ) ) {
			if ( $data ) {
				$data = __( 'Yes', 'linear' );
			} else {
				$data = __( 'No', 'linear' );
			}
		} else {
			$data = strval( $data );
		}
		echo wp_kses( $data, array( 'br' => array() ) );
	}

	/**
	 * Returns label for specific data field.
	 * 
	 * @param  string $key     Key for data field containing API constant.
	 * @param  mixed  $default Optional value.
	 * @return string
	 */
	public function get_data_label( $key, $default = null) {
		$slug = $this->get_data( $key, $default );
		return self::$linear->get_slug_label( "$slug@$key" );
	}
	
	/**
	 * Returns label array for data field with API constants.
	 * 
	 * @param  string $key Key for data field containing array of API constants.
	 * @return string[]
	 */
	public function get_data_labels( $key ) {
		$labels = array_map(
			function ( $slug ) use ( $key ) {
				return self::$linear->get_slug_label( "$slug@$key" );
			},
			$this->get_data( $key )
		); 
		return $labels;
	}

	/**
	 * Returns array of data fields.
	 * Resulting array can be filtered by $filter.
	 * If $filter is callback it will be used for array_filter.
	 * If $filter is false array_filter will not be used.
	 * If $filter is true apply array_filter for empty().
	 * 
	 * @param  boolean|callback $filter Optional filter callback or boolean.
	 * @param  string[]       $keys Keys for specific data fields.
	 * @return array
	 */
	public function get_data_group( $keys, $filter = false ){
		$group = array();
		foreach ( $keys as $key ) {
			if ( $this->has_data( $key ) ) {
				$group[$key] = $this->get_data( $key );
			}
		}

		if ( $filter === false ) {
			return $group;
		} elseif ( $filter === true ) {
			return array_filter( $group );
		}
		return array_filter( $group, $filter );
	}

	/**
	 * Returns price string of specific data fields.
	 * To calculate price based on two different data fields you can pass
	 * array of two keys and one operator +, -, * or / as a $key parameter.
	 * Price for m² example: the_price_data( array( 'debtFreePrice', '/', 'area'), '/ m²' )
	 * 
	 * @param  string|array $key    Key/s for data fields containig price numbers.
	 * @param  string       $suffix Optional string that will be added on the end of price string.
	 * @param  float|int    $suffix Optional factor for price value.
	 * @return string
	 */
	public function get_price_data( $key, $suffix = '', $factor = 1 ) {
		if( is_array( $key ) && 3 === count( $key ) ) {
			$value = array_map(
				function ( $value ) {
					if ( $this->has_data( $value ) ) {
						$value = $this->get_data( $value );
					}
					if( is_string( $value ) ) {
						$value = str_replace( array(',', ' '), array('.', ''), $value );
					}
					if( is_numeric( $value ) ) {
						return $value;
					}
					return 0;
				},
				array( 
					$key[0],
					$key[2]
				)
			);
			if ( is_numeric( $value[0] ) && is_numeric( $value[1] ) ) {
				switch( $key[1] ) {
					case '+':
						$value = $value[0] + $value[1];
						break;
					case '-':
						$value = $value[0] - $value[1];
						break;
					case '*':
						return $value = $value[0] * $value[1];
						break;

					case '/':
						$value = $value[0] / $value[1];
						break;
				}
			}  
		} else {
			if ( is_numeric( $key ) ) {
				$value = $key;
			} elseif ( is_numeric( str_replace( array(',', ' '), array('.', ''), $key ) ) ) {
				$value = str_replace( array(',', ' '), array('.', ''), $key );
			} else {
				$value = $this->get_data( $key, 0 );
				if( is_string( $value ) ) {
					$value = str_replace( array(',', ' '), array('.', ''), $value );
				}
			}
		}

		if ( ! is_numeric( $value ) ) {
			return strval( $value );
		}

		$value = round( $value * $factor, 2);
		if( ! empty( $suffix ) && $suffix[0] !== ' ' ) {
			$suffix = ' ' . $suffix;
		}
		return self::$linear->number_format( $value ) . '&nbsp;'. '€' . $suffix;
	}

	/**
	 * Display escaped price string of specific data fields.
	 * To calculate price based on two different data fields you can pass
	 * array of two keys and one operator +, -, * or / as a $key parameter.
	 * Price for m² example: the_price_data( array( 'debtFreePrice', '/', 'area'), '/ m²' )
	 * 
	 * @param string|array $key    Key/s for data fields containig price numbers.
	 * @param string       $suffix Optional string that will be added on the end of price string.
	 * @param float|int    $suffix Optional factor for price value.
	 */
	public function the_price_data( $key, $suffix = '', $factor = 1 ) {
		echo esc_html( $this->get_price_data( $key, $suffix, $factor ) );
	}

	/**
	 * Gets next available presentation by type if it exists.
	 * 
	 * @param  string $type API constant (eg: PUBLIC).
	 * @return string|null
	 */
	public function get_next_presentation( $type ) {
		$next_presentation = null;
		if ( $this->has_data( 'presentations' ) ) {
			foreach ( $this->get_data( 'presentations' ) as $presentation ) {
				if ( $presentation['type'] === $type ) {
					$date = new DateTime( $presentation['date'] );
					$end_time = new DateTime( $presentation['endTime'] );
					$combined_date = new DateTime($date->format('Y-m-d') . ' ' .$end_time->format('H:i:s'));

					if ( strtotime('today') <= $combined_date->getTimestamp() ) {
						if ( empty( $next_presentation ) || strtotime( $presentation['date'] ) <= strtotime( $next_presentation['date'] ) ) {
							$next_presentation = $presentation;
						}
					}
				}
			}
		}
		return $next_presentation;
	}

	/**
	 * Display next available presentation date and time.
	 * If presentation doesn't exist display default value.
	 * 
	 * @param  string $type    API constant (eg: PUBLIC).
	 * @param  string $default Oprional default string.
	 * @return string|null
	 */
	public function the_next_presentation( $type, $default = null ){
		$presentation = $this->get_next_presentation( $type );
		if ( empty( $presentation ) ) {
			$presentation = $default;
		} else {
			$presentation =  sprintf(
				'%s %s&nbsp;-&nbsp;%s',
				str_replace ( ' ', '&nbsp;', date_i18n( get_option( 'date_format' ), strtotime( $presentation['date'] ) ) ),
				str_replace ( ' ', '&nbsp;', date_i18n( get_option( 'time_format' ), strtotime( $presentation['startTime'] ) ) ),
				str_replace ( ' ', '&nbsp;', date_i18n( get_option( 'time_format' ), strtotime( $presentation['endTime'] ) ) )
			);
		}
		echo esc_html( $presentation );
	}

	/**
	 * Return listing realtor data by key.
	 * 
	 * @param  string $key Key for realtor data field.
	 * @return mixed
	 */
	public function get_realtor_data( $key ){
		if ( $this->has_data( 'realtor' ) ) {
			if ( isset( $this->data['realtor'][$key] ) ) {
				return $this->data['realtor'][$key];
			}
		}
		return null;
	}


	/**
	 * Returns realtor logo url.
	 */
	public function get_realtor_logo_url(){
		if ( wp_attachment_is_image( self::$linear->get_option( 'company_logo' ) ) ) {
			return wp_get_attachment_url( self::$linear->get_option( 'company_logo' ) );
		}
		return empty( $this->get_realtor_data( 'companyLogo' ) ) ? null : $this->get_image_url( $this->get_realtor_data( 'companyLogo' ) );
	}

	/**
	 * Display escaped realtor logo url.
	 */
	public function the_realtor_logo_url(){
		echo esc_url( $this->get_realtor_logo_url() );
	}

	/**
	 * Display listing realtor data by key.
	 * 
	 * @param  string $key Key for realtor data field.
	 */
	public function the_realtor_data( $key ){
		echo esc_html( strval( $this->get_realtor_data( $key ) ) );
	}

	/**
	 * Returns featured image url.
	 * 
	 * @return string
	 */
	public function get_featured_url() {
		if ( $this->has_data( 'images' ) ) {
			$image = $this->data['images'][0];
		} elseif ( $this->has_data( 'thumbnails' ) ) {
			$image = $this->data['thumbnails'][0];
		} else {
			$image = '';
		}
		return $this->get_image_url( $image );
	}

	/**
	 * Display featured image escaped url.
	 * 
	 * @return string
	 */
	public function the_featured_url() {
		echo esc_url( $this->get_featured_url() );
	}

	/**
	 * Returns youtube video thumbnail url.
	 * Size that must exist:       480x360 hqdefault.jpg
	 * Sizes that might not exist: 640x480 sddefault.jpg, 1280x720 hq720.jpg, 1920x1080 maxresdefault.jpg
	 * 
	 * @param  string $video_url Youtube video url.
	 * @param  string $size      Optional size for youtube video thumbnail.
	 * @return string
	 */
	public function get_youtube_video_image_url( $video_url, $size = 'hqdefault.jpg' ) {
		$url = 'https://img.youtube.com/vi/';
		if ( preg_match('/(?:youtube(?:-nocookie)?\.com\/(?:[^\/]+\/.+\/|(?:v|e(?:mbed)?)\/|.*[?&]v=)|youtu\.be\/)([^"&?\/ ]{11})/', $video_url, $match ) ) {
			$url .= $match[1];
			if ( $size !== 'hqdefault.jpg' && 200 != wp_remote_retrieve_response_code( wp_remote_get( "$url/$size" ) ) ) {
				$size = 'hqdefault.jpg';
			}
		}
		return "$url/$size";
	}

	/**
	 * Display escaped youtube video thumbnail url.
	 * Size that must exist:       480x360 hqdefault.jpg
	 * Sizes that might not exist: 640x480 sddefault.jpg, 1280x720 hq720.jpg, 1920x1080 maxresdefault.jpg
	 * 
	 * @param  string $video_url Youtube video url.
	 * @param  string $size      Optional size for youtube video thumbnail.
	 * @return string
	 */
	public function the_youtube_video_image_url( $video_url, $size = 'hqdefault.jpg' ) {
		echo esc_url( $this->get_youtube_video_image_url( $video_url, $size ) );
	}

	/**
	 * Display escaped youtube video embed url.
	 * 
	 * @param  string $video_url Youtube video url.
	 * @param  array  $query     Optional url parameters.
	 * @return string
	 */
	public function the_youtube_video_embed_url( $video_url, $query = array( 'autoplay' => 1 ) ) {
		if ( preg_match('/(?:youtube(?:-nocookie)?\.com\/(?:[^\/]+\/.+\/|(?:v|e(?:mbed)?)\/|.*[?&]v=)|youtu\.be\/)([^"&?\/ ]{11})/', $video_url, $match ) ) {
			$url = 'https://www.youtube.com/embed/' . $match[1];
			$query = http_build_query( $query );
			if ( ! empty( $query ) ) {
				$url .= "?$query";
			}
		}
		echo esc_url( $url );
	}
}
