<?php
/**
 * Fetching content from api functionality
 *
 * @package Linear
 */

/**
 * Middleware for handling import of data
 */
class Linear_Middleware {

	protected static $api_url;
	protected static $api_key;
	protected static $company_id;
	protected static $listings_page;
	protected static $api_version;
	protected static $plugin_version;
	protected static $linear;

	public function __construct() {
		if ( is_null( self::$linear ) ) {
			self::$linear = Linear::get_instance();
		}
		
		self::$api_url 			= self::$linear->get_option( 'api_url' ) ?: '';
		self::$api_key 			= self::$linear->get_option( 'api_key' ) ?: '';
		self::$company_id 		= self::$linear->get_option( 'company_id' ) ?: '';
		self::$listings_page 	= self::$linear->get_option( 'listings_page' ) ?: 0;
		self::$api_version 		= ( self::$linear->get_option( 'api_key' ) ? 'v2' : 'v1' );
		self::$plugin_version 	= defined( 'LINEAR_VERSION' ) ? LINEAR_VERSION : '';
	}

	/**
	 * Initialize the middleware
	 * 
	 * Check current data and if it needs to be updated
	 */
	public function init(){
		if( is_admin() ){

			add_action('admin_head', function(){
				$this->run( false, [] );
			});

		} else {

			add_action('wp', function(){
				global $post;

				if( isset( $post->ID) && in_array( $post->ID, [
					intval( self::$linear->get_option( 'listings_page' ) ?: 0 ),
					intval( self::$linear->get_option( 'rentals_page' ) ?: 0 ),
					intval( self::$linear->get_option( 'workplace_page' ) ?: 0 )
				] ) || has_block( 'linear/listings', $post ) ){
					$this->pre_run( $post->ID, [] );
				}
			});

		}
	}

	/**
	 * Small wrapper for reusing Gutenberg conditional code.
	 * Will be removed once the old fetching system is removed.
	 * 
	 * @param int $post_id
	 */
	private function pre_run( $post_id = '' ){
		if( !$post_id ){
			return;
		}

		if ( function_exists( 'has_blocks' ) && has_blocks( $post_id ) ) {
			if( !get_transient( $this->get_transient_name() ) ){
				$this->run( false );
			}
		}
	}

	/**
	 * Get the listings
	 * 
	 * @return array|null
	 */
	public function get_all(){
		return $this->run( true );
	}

	public function get_all_by_type( $parse_type ){
		if( !$parse_type ){
			return [];
		}

		$parsed_listings = [];
		$listings = $this->run( true );

		if( !is_array( $listings ) ){
			return $parsed_listings;
		}

		foreach( $listings as $listing ){
			if( $parse_type === 'APARTMENTS' ){

				$additional_types = [
					'APARTMENTS',
					'PLOTS',
					'FARMS',
					'GARAGES',
					'NEWLY_CONSTRUCTED',
					'VACATION_APARTMENT'
				];

				if( 
					isset( $listing['rawDataForFiltering']['productGroup'] ) &&
					in_array( $listing['rawDataForFiltering']['productGroup'], $additional_types )
				){
					array_push( $parsed_listings, $listing );
				}
			} else {
				// Easier listing
				if( 
					isset( $listing['rawDataForFiltering']['productGroup'] ) &&
					$listing['rawDataForFiltering']['productGroup'] === $parse_type 
				){
					array_push( $parsed_listings, $listing );
				}
			}
		}

		return $parsed_listings;
	}

	/**
	 * Get the listings
	 * 
	 * @param string $id Listing id
	 * 
	 * @return array|null
	 */
	public function get_single( $id = '' ){
		$listings_data = $this->run( true );

		// Validate listings data
		if( !$listings_data ){
			return null;
		}
		
		// Iterate each until we get a response
		foreach( $listings_data as $single ){
			if( $single['id'] === $id ){
				return $single;
			}
		}

		// Fallback
		return null;
	}

	/**
	 * Override transient check and force update
	 */
	public function update(){
		// Fetch new listings
		$listings_data = call_user_func( 'fetch_listings_api_' . self::$api_version );

		// If we have a valid response, set 10 minute transient
		if( $listings_data ){
			return set_transient( $this->get_transient_name(), $listings_data, 300 );
		}

		return false;
	}

	/**
	 * Run the middleware.
	 * 
	 * $param bool $return if we want a response
	 */
	public function run( $return = false ){
		// Transient name not set
		if( !$this->get_transient_name() ){
			return new WP_Error( 'error', __( "API middleware not getting a transient name to use", "linear" ) );
		}

		// We have a response
		if( $response = get_transient( $this->get_transient_name() ) ){
			if( $return ){
				return $response;
			}

			// End run
			return;
		}

		// Fetch new listings
		$fetch_listing_method = 'fetch_listings_api_' . self::$api_version;
		$listings_data = $this::$fetch_listing_method();

		// If we have a valid response, set 10 minute transient
		if( $listings_data && !is_wp_error( $listings_data ) ){
			set_transient( $this->get_transient_name(), $listings_data, 600 );
		}

		if( $return ){
			return $listings_data;
		}

		return null;
	}

	/**
	 * Helper for fetching the transient name
	 */
	private function get_transient_name() {
		// Validate that we have all necessary values
		if( !self::$api_version || !self::$api_url || !self::$company_id || !self::$plugin_version ){
			return '';
		}

		$listing_page_url = get_permalink( self::$listings_page );
		if( !$listing_page_url ){
			$listing_page_url = self::$listings_page;
		}

		return 
			"linear_middleware_api_" . 
			self::$api_version . 
			"_" .  
			substr( get_locale(), 0, 2 ) .
			"_" . 
			hash('crc32', self::$api_url . self::$company_id . (isset( self::$api_key ) ? !self::$api_key : '' )  . self::$plugin_version . $listing_page_url, false);
	}

	/**
	 * Fetching v1 data
	 * 
	 * @param array $parameters URL-parameters for request
	 * 
	 * @return array|WP_Error
	 */
	private function fetch_listings_api_v1() {

		// Setup params
		$parameters = [];
		$parameters['company'] = self::$company_id;
		$parameters['migratedListings'] = "1";
		$parameters['withoutTranslation'] = "true";

		$endpoints = [
			"",
			"RENT_APARTMENT",
			"BUSINESS_PREMISES"
		];

		$fetch_headers = [];
		$listings_data = [];

		// Iterate through all the different data-types and merge the results
		foreach( $endpoints as $endpoint ){

			// Setup for specific data-type fetchingf
			if( $endpoint !== '' ){
				$parameters['productGroup'] = $endpoint;
			}

			// We have to fetch 24 listings at a time with pagination, end at no new results.
			// Limit fetch to max 480 listings aka. 20 pages
			for ( $page = 1; $page <= 20; $page++ ) {

				$parameters['page'] = $page;
				$fetch_url = self::$api_url . '/v1/dixu-listings?' . http_build_query( $parameters );
				
				$response = wp_remote_get( $fetch_url, $fetch_headers );

				// If failed response
				if( is_wp_error( $response ) ){
					return new WP_Error( 'error', __( "API v1 middleware failed to fetch data", "linear" ) );
					break;
				}

				$request_body = json_decode( wp_remote_retrieve_body( $response ), true );

				// If missing "listings"
				if ( !isset( $request_body['listings'] ) ) {
					return new WP_Error( 'error', __( "API v1 middleware invalid fetched format of data", "linear" ) );
					break;
				}

				// If "listings" is empty
				if( empty( $request_body['listings'] ) ){
					break;
				}

				$listings_data = array_merge( $listings_data, $request_body['listings'] );
			}
		}

		// Check if we have any results
		if( empty( $listings_data ) ){
			return [];
		}

		$extended_listings_data = [];
		// Iterate each result individually to get all the data
		foreach( $listings_data as $key => $listings_data ){
			$singular_data = $this->fetch_singular_data_v1( $listings_data['id'] );

			if( $singular_data ){
				$extended_listings_data[$key] = $singular_data;
			}
		}

		return $this->cleanup_data( $extended_listings_data );
	}

	private function fetch_singular_data_v1( $id ){

		if( !$id ){
			return null;
		}

		$fetch_url = self::$api_url . '/dixu-listings/' . $id ;

		$response = wp_remote_get( $fetch_url );

		// If failed response
		if( is_wp_error( $response ) ){
			return [];
		}

		// In case we get a error
		if( isset( $request_body['type'] ) ){
			return new WP_Error( 'error', $request_body['plaintext'] );
		}

		$request_body = json_decode( wp_remote_retrieve_body( $response ), true );

		return $request_body;
	}

	/**
	 * Fetching v2 data
	 * 
	 * @param array $parameters URL-parameters for request
	 * 
	 * @return array|WP_Error
	 */
	private function fetch_listings_api_v2() {

		// Setup params
		$parameters = [];
		$parameters['companyId'] = self::$company_id;
		$parameters['migratedListings'] = "1";
		$parameters['translate'] = 'fi';
		//$parameters['translate'] = substr( get_locale(), 0, 2 );
		// $parameters['withoutTranslation'] = "true";

		$fetch_url = self::$api_url . '/v1/dixu-listings/all?' . http_build_query( $parameters );
		$fetch_headers = [
			'headers' => [
				'Accept' => 'application/json',
				'Authorization' => self::$api_key,
			],
			'timeout' => 60,
		];

		$response = wp_remote_get( $fetch_url, $fetch_headers );

		if ( is_wp_error( $response ) ) {
			return new WP_Error( 'error', __( "API v2 middleware failed to fetch data", "linear" ) );
		}

		$request_body = json_decode( wp_remote_retrieve_body( $response ), true );

		// No results in response
		if ( !isset( $request_body ) ) {
			return new WP_Error( 'error', __( "API v2 middleware failed in parsing response data", "linear" ) );
		}

		// If error message
		if( is_string( $request_body ) ){
			return new WP_Error( 'error', $request_body );
		}

		// In case we get a error
		if( isset( $request_body['type'] ) ){
			return new WP_Error( 'error', $request_body['plaintext'] );
		}

		return $this->cleanup_data( $request_body );
	}

	/**
	 * Cleanup data for safer/easier handling
	 * 
	 * @params array $listings_data Listings data
	 * 
	 * @return $array Listings data
	 */
	private function cleanup_data( $listings_data = [] ){
		
		$listings_data = $this->sanitize_data( $listings_data );
		$listings_data = $this->generalize_data( $listings_data );

		return $listings_data;
	}

	/**
	 * Brought in from wp-qa.com/wordpress-sanitize-array
	 */
	private function recursive_sanitize_textarea_field( $array ) {
		foreach ( $array as $key => &$value ) {
			if ( is_array( $value ) ) {
				$value = $this->recursive_sanitize_textarea_field( $value );
			}
			else {
				$value = sanitize_textarea_field( $value );
			}
		}
	
		return $array;
	}

	/**
	 * Cleanup empty/null values from data
	 * 
	 * @params array $listings_data Listings data
	 * @params array $default default data in case of empty data
	 * 
	 * @return $array Listings data
	 */
	private function sanitize_data( $listings_data = [], $default = [] ){

		$cleaned_data = [];

		if( $listings_data ){
			// Start iterating items
			foreach( $listings_data as $listing ){
				$cleaned_single = [];

				if( $listing ){
					// Iterate single items
					foreach( $listing as $key => $value ){
						if( $value !== [] && $value !== null ){
							$cleaned_single[$key] = $value;

							// Sanitize per data type
							if ( is_array( $value ) ) {
								$cleaned_single[$key] = $this->recursive_sanitize_textarea_field( $value );
							} else if( is_string( $value ) ) {
								$cleaned_single[$key] = sanitize_textarea_field( $value );
							} else if( is_int( $value ) || strval( intval( $value) ) === $value ) {
								$cleaned_single[$key] = intval( $value );
							} else {
								$cleaned_single[$key] = rest_sanitize_boolean( $value );
							}

						}
					}
				}

				// If we have the most important data, populate
				if( $cleaned_single['id'] ){
					array_push( $cleaned_data, $cleaned_single);
				}
			}
		}

		if( !$cleaned_data ){
			return $default;
		}

		return $cleaned_data;
	}

	/**
	 * Different API-endpoints return data in slightly different formats.
	 * Lets use this generalizer to generalize the data to a single format
	 * for easier frontend use.
	 * 
	 * @params array $listings_data Listings data
	 * 
	 * @return $array Listings data
	 */
	private function generalize_data( $listings_data = [] ){
		$this->data_handler = self::$linear->get_data_handler();
		
		return $this->data_handler->generalize_listings( $listings_data );
	}
}