<?php
/**
 * Displays plugin settings page and takes care of saving.
 * 
 * @package Linear
 */

class Linear_Settings {

	/**
	 * Core plugin class instance.
	 *
	 * @var Linear
	 */
	protected static $linear;
	protected $api_key;

	/**
	 * Defines class properties.
	 */
	public function __construct() {
		if ( is_null( self::$linear ) ) {
			self::$linear = Linear::get_instance();
		}

		$this->api_version = ( self::$linear->get_option( 'api_key' ) ? 'v2' : 'v1' );
	}
 
	/**
	 * Enqueue necessary scripts for plugin settings page.
	 */
	function admin_page_scripts() {
		if ( ! did_action( 'wp_enqueue_media' ) ) {
			wp_enqueue_media();
		}
	}

	/**
	 * Adds settings page in admin dashboard.
	 */
	public function add_plugin_page() {
		$title = 'Linear';

		if( $this->api_version === 'v1'){
			$title = $title . " <span class='update-plugins count-1'><span class='plugin-count' aria-hidden='true'>1</span><span class='screen-reader-text'>" . __('Update your API credentials', 'linear') . "</span></span>";
		}

		// Init admin page
		add_menu_page(
			'Linear settings',
			$title,
			'manage_options',
			'linear-settings',
			'',
			'dashicons-admin-multisite',
			81
		);

		// Create link to main option page
		add_submenu_page(
			'linear-settings',
			__( 'Settings', 'linear' ),
			__( 'Settings', 'linear' ),
			'manage_options',
			'linear-settings',
			array( $this, 'create_admin_page' )
		);

		// Advanced usage page
		add_submenu_page(
			'linear-settings',
			__( 'Advanced usage', 'linear' ),
			__( 'Advanced usage', 'linear' ),
			'manage_options',
			'linear-guide',
			array( $this, 'create_admin_guide' )
		);

		// Advanced usage page
		add_submenu_page(
			'linear-settings',
			__( 'Feedback', 'linear' ),
			__( 'Feedback', 'linear' ),
			'manage_options',
			'linear-feedback',
			array( $this, 'create_admin_feedback' )
		);
	}

	/**
	 * Create settings page.
	 */
	public function create_admin_page() {
		?>
		<div class="wrap">
			<h2><?php esc_html_e( 'Linear settings', 'linear' ); ?></h2>
			<p></p>
			<?php settings_errors(); ?>

			<form method="post" action="options.php">
				<?php
					settings_fields( 'option_group' );
					do_settings_sections( 'linear-settings-admin' );
					submit_button();
				?>
			</form>
		</div>
		<?php 
	}

	/**
	 * Create advanced usage page
	 */
	public function create_admin_guide() { 
		?>
		<div class="wrap">
			<h1><?php _e( 'Advanced usage', 'linear' ); ?></h1>
			
			<?php _e( 'The Linear plugin contains quite a bit of logic under the hood and provides ease of access to its features.', 'linear' ); ?>

			<ul>
				<li><a href="#shortcode"><?php _e( 'Shortcodes', 'linear' ); ?></a></li>
				<li><a href="#restapi"><?php _e( 'REST-API', 'linear' ); ?></a></li>
			</ul>

			<?php
				// Main guides
				echo $this->shortcode_guide( "shortcode" );
				echo $this->rest_api_guide( "restapi" );
			?>

		</div>
		<?php
	}

	/**
	 * Create advin feedback
	 */
	public function create_admin_feedback() { 
		?>
		<div class="wrap">
			<h1><?php _e( 'Feedback and suggestions', 'linear' ); ?></h1>
			
			<?php echo('
				<p>' . __('If you feel like there is some feature/filter missing or something should be different, please let us know as we actively try to develop this plugin and are open for suggestions and improvements.', 'linear') . '</p>
				<p>' . __('You can share your thoughts via email to <a href="mailto:info@linear.fi">info@linear.fi</a>', 'linear') . '</p>
			'); ?>

		</div>
		<?php
	}

	/**
	 * Add and register settings sections and fields.
	 */
	public function page_init() {
		register_setting(
			'option_group',
			'linear_settings',
			array( $this, 'sanitize' )
		);

		add_settings_section(
			'setting_section',
			__( 'Settings', 'linear' ),
			array( $this, 'section_info' ),
			'linear-settings-admin'
		);

		add_settings_field(
			'company_id',
			__( 'Company ID', 'linear' )  . ' <span style="color:red">*</span>',
			array( $this, 'company_id_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		add_settings_field(
			'api_url',
			__( 'API url', 'linear' ) . ' <span style="color:red">*</span>',
			array( $this, 'api_url_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		add_settings_field(
			'api_key',
			__( 'API key', 'linear' ) . '',
			array( $this, 'api_key_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		add_settings_field(
			'terms_of_service',
			__( 'Terms of service', 'linear' ) . ' <span style="color:red">*</span>',
			array( $this, 'terms_of_service_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		add_settings_field(
			'contact_url',
			__( 'Contact request URL', 'linear' ),
			array( $this, 'contact_url_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		add_settings_field(
			'company_logo',
			__( 'Company Logo', 'linear' ),
			array( $this, 'company_logo_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		add_settings_field(
			'theme_fonts',
			__( 'Use theme fonts', 'linear' ),
			array( $this, 'theme_fonts_callback' ),
			'linear-settings-admin',
			'setting_section'
		);
		
		add_settings_field(
			'loan_request',
			__( 'Hide loan request button', 'linear' ),
			array( $this, 'loan_request_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		/*
		add_settings_field(
			'leadoo_bot',
			__( 'Use Leadoo InPageBot', 'linear' ),
			array( $this, 'leadoo_bot_callback' ),
			'linear-settings-admin',
			'setting_section'
		);
		*/

		add_settings_field(
			'search_options',
			__( 'Default search options', 'linear' ),
			array( $this, 'search_options_callback' ),
			'linear-settings-admin',
			'setting_section'
		);

		foreach ( array(
			array( 
				'id' => 'primary_color',
				'title' => __( 'Primary color', 'linear' ),
				'default_color' => '#1890ff'
			),
			array(
				'id' => 'solid_color',
				'title' => __( 'Solid color', 'linear' ),
				'default_color' => '#096dd9'
			),
			array(
				'id' => 'outline_color',
				'title' => __( 'Outline color', 'linear' ),
				'default_color' => '#0050b3'
			)
		) as $props ) {
			add_settings_field(
				$props['id'],
				$props['title'],
				array( $this, 'select_color_callback' ),
				'linear-settings-admin',
				'setting_section',
				$props
			);
		}

		add_settings_field(
			'listings_page',
			__( 'Listing page', 'linear' ) . ' <span style="color:red">*</span>',
			array( $this, 'select_page_callback' ),
			'linear-settings-admin',
			'setting_section',
			array( 'id' => 'listings_page' )
		);

		add_settings_field(
			'rentals_page',
			__( 'Rentals page', 'linear' ),
			array( $this, 'select_page_callback' ),
			'linear-settings-admin',
			'setting_section',
			array( 'id' => 'rentals_page' )
		);

		add_settings_field(
			'workplace_page',
			__( 'Business premises page', 'linear' ),
			array( $this, 'select_page_callback' ),
			'linear-settings-admin',
			'setting_section',
			array( 'id' => 'workplace_page' )
		);
	}

	/**
	 * Returns sanitized and validated settings values.
	 * 
	 * @return array
	 */
	public function sanitize( $input ) {
		$sanitary_values = array();
		foreach ( $input as $name => $value ) {
			if ( 'search_options' === $name ) {
				// Remove unnecessary multiple spaces
				$value = preg_replace( '/\h+/', ' ', $value );
				// Remove empty lines of text and split to array of lines
				$value = preg_split( "/(^[\r\n]*|[\r\n]+)[\s\t]*[\r\n]+/", $value );
				// Sanitize text fields
				$value = array_map( 'sanitize_text_field', $value );
			} elseif ( 'api_url' === $name ) {
				$value = esc_url_raw( rtrim( strtok( $value, '?'), '/') );
			} elseif ( 'api_key' === $name ) {
				$value = sanitize_text_field( $value );
			} elseif ( 'contact_url' === $name ) {
				$value = esc_url_raw( $value );
			} elseif ( in_array( $name, array( 'outline_color', 'solid_color', 'primary_color' ), true ) ){
				$value = sanitize_hex_color( $value );
			} elseif ( 'theme_fonts' === $name || 'terms_of_service' === $name || 'loan_request' === $name ) {
				$value = isset( $value ) && true == $value ? 1 : 0;
			} elseif ( in_array( $name, array( 'listings_page', 'company_id', 'company_logo', 'rentals_page', 'workplace_page', ), true ) && is_numeric( $value ) ) {
				$value = absint( $value );
				/*
			} elseif ( in_array( $name, array( 'leadoo_bot' ), true ) ) {
				$value = sanitize_text_field( $value );
				*/
			} else {
				// Jump to next input if name is unknown and value is not sanitized.
				continue;
			}
			$sanitary_values[$name] = $value;
		}

		return $sanitary_values;
	}

	public function linear_settings_pre_update_callback( $new_value, $old_value ) {
		if (! isset( $old_value['company_id'] ) || ( isset( $new_value['company_id'] ) && $new_value['company_id'] === $old_value['company_id'] ) ) {
			$company_id = self::$linear->get_option( 'company_id' );
		} else {
			$company_id = $new_value['company_id'];
			// Force refresh of leadoo_bot data if company_id changed.
			/*
			if ( isset( $old_value['leadoo_bot'] ) ) {
				unset( $old_value['leadoo_bot'] );
				$new_value['leadoo_bot'] = 1;
			}
			*/
		}

		/*
		if ( ( isset( $new_value['leadoo_bot'] ) xor isset( $old_value['leadoo_bot'] ) ) ) {
			if ( isset( $new_value['leadoo_bot'] ) ) {
				unset( $new_value['leadoo_bot'] );
				$request = self::$linear->get_option( 'api_url' ) . '/plugin/company/' . (string) $company_id;
				$request = wp_remote_get( $request );
				if ( is_wp_error( $request ) ) {
					error_log( $request->get_error_message() );
				} else {
					$request = json_decode( wp_remote_retrieve_body( $request ), true );
					if ( isset( $request['pluginLeadooUrl'] ) && ! empty( $request['pluginLeadooUrl'] ) ) {
						$new_value['leadoo_bot'] = sanitize_text_field( $request['pluginLeadooUrl'] );
					} else {
						add_settings_error( 'linear_settings', "leadoo_bot_error", esc_html( 'Leadoo InPageBot is not currently confiugred on your Linear profile.', 'linear' ), 'warning' );
					}
				}
			} else {
				unset( $new_value['leadoo_bot'] );
			}
		}
		*/

		// Each listing page must be different page, remove duplicates.
		$listing_pages = array();
		foreach ( array( 'listings_page', 'rentals_page', 'workplace_page' ) as $key ) {
			if ( isset( $new_value[$key] ) && $new_value[$key] ) {
				if ( in_array( $new_value[$key], $listing_pages ) ) {
					unset( $new_value[$key] );
				} else {
					$listing_pages[] = $new_value[$key];
				}
			}
		}

		return $new_value;
	}

	public function linear_settings_updated_callback( $old_value, $new_value ) {
		$flush_rewrite_rules = false;
		foreach ( array( 'listings_page', 'rentals_page', 'workplace_page' ) as $key ) {
			if ( isset( $old_value[$key] ) && $old_value[$key] != $new_value[$key] ) {
				$flush_rewrite_rules = true;
			}
		}
		if ( $flush_rewrite_rules ) {
			Linear::listings_page_rewrite_rules( true );
		}
	}

	/**
	 * Display info for settings section.
	 */
	public function section_info() {
		
	}

	/**
	 * Display company_id field.
	 */
	public function company_id_callback() {
		printf(
			'<input class="regular-text" type="text" name="linear_settings[company_id]" id="company_id" value="%s" required>',
			esc_attr( self::$linear->get_option( 'company_id' ) )
		);
	}

	/**
	 * Display api_url field.
	 */
	public function api_url_callback() {
		printf(
			'<input class="regular-text" type="url" name="linear_settings[api_url]" id="api_url" value="%s" required>',
			esc_attr( self::$linear->get_option( 'api_url' ) )
		);
	}

	/**
	 * Display api_key field.
	 */
	public function api_key_callback() {
		printf(
			'<input class="regular-text" type="text" name="linear_settings[api_key]" id="api_key" value="%s">',
			esc_attr( self::$linear->get_option( 'api_key' ) )
		);
	}

	/**
	 * Display contact_url field.
	 */
	public function contact_url_callback() {
		printf(
			'<input class="regular-text" type="url" name="linear_settings[contact_url]" id="contact_url" value="%s">',
			esc_attr( self::$linear->get_option( 'contact_url' ) )
		);
	}

	/**
	 * Display company_logo field.
	 */
	public function company_logo_callback() {
		if ( $image_url = wp_get_attachment_url( self::$linear->get_option( 'company_logo' ) ) ) :
			?>
			<button data-upload="company_logo" style="height:auto; min-height:32px; line-height:0; padding:10px" type="button" class="button media-button  select-mode-toggle-button">
				<img height="100" src="<?php echo esc_url( $image_url ); ?>" />
			</button>
			<p><a href="#" data-remove="company_logo"><?php esc_html_e( 'Remove image', 'linear' ); ?></a></p>
			<input type="hidden" name="linear_settings[company_logo]" value="<?php echo esc_attr( self::$linear->get_option( 'company_logo' ) ); ?>">
			<?php
		else :
			?>
			<button data-upload="company_logo" style="height:auto; min-height:32px; line-height:0; padding:10px" type="button" class="button media-button  select-mode-toggle-button">
				<?php esc_html_e( 'Upload image', 'linear' ); ?>
			</button>
			<p><a href="#" data-remove="company_logo" style="display:none;"><?php esc_html_e( 'Remove image', 'linear' ); ?></a></p>
			<input type="hidden" name="linear_settings[company_logo]">
			<?php
		endif;
	}

	/**
	 * Display select color fields.
	 * 
	 * @param array $args Array of arguments, must contain 'id' and 'default_color' keys with values.
	 */
	public function select_color_callback( $args ) {
		extract( $args );
		printf(
			'<input data-default-color="%2$s" type="text" name="linear_settings[%1$s]" id="%1$s" value="%3$s">',
			$id,
			$default_color,
			esc_attr( self::$linear->get_option( $id, $default_color ) )
		);
	}

	/**
	 * Display select page fields.
	 * 
	 * @param array $args Array of arguments, must contain 'id' key with value.
	 */
	public function select_page_callback( $args ) {
		extract( $args );

		ob_start();

			wp_dropdown_pages(
				array(
					'selected'         => self::$linear->get_option( $id ),
					'name'             => "linear_settings[$id]",
					'show_option_none' => __( 'None', 'linear' ),
					'sort_column'      => 'post_title',
				)
			);
		
		$renderable = ob_get_clean();

		if( $id === 'listings_page' ){
			echo str_replace( "<select ", "<select required ", $renderable );
		} else {
			echo $renderable;
		}
	}

	/**
	 * Display search_options field.
	 */
	public function search_options_callback() {
		printf(
			'<textarea style="white-space: pre-wrap;" rows="5" name="linear_settings[search_options]" id="search_options">%s</textarea><p class="description" id="tagline-description">%s</p>',
			empty( self::$linear->get_option( 'search_options' ) ) ?  '' : esc_textarea( implode( PHP_EOL, self::$linear->get_option( 'search_options' ) ) ),
			__( 'Each line of text will be used as one of default search options. You can use either city names, districts or addresses, eg. "Helsinki, Espoo, Vantaa"', 'linear' )
		);
	}

	/**
	 * Display theme_fonts field.
	 */
	public function theme_fonts_callback() {
		$theme_fonts = self::$linear->get_option( 'theme_fonts' );
		printf(
			'<input type="checkbox" id="theme_fonts" name="linear_settings[theme_fonts]" value="1" %s /> %s',
			checked( '1', $theme_fonts, false ),
			__( 'Check to use current WordPress theme fonts instead of Linear plugin fonts.', 'linear' )
		);
	}

	/**
	 * Display loan_request field.
	 */
	public function loan_request_callback() {
		printf(
			'<input type="checkbox" id="loan_request" name="linear_settings[loan_request]" value="1" %s />',
			checked( '1', self::$linear->get_option( 'loan_request' ), false )
		);
	}

	/**
	 * Display leadoo_bot field.
	 */
	/*
	public function leadoo_bot_callback() {
		printf(
			'<input type="checkbox" id="leadoo_bot" name="linear_settings[leadoo_bot]" value="%s" %s /> %s',
			self::$linear->get_option( 'leadoo_bot', 1 ),
			checked( true, (bool) self::$linear->get_option( 'leadoo_bot' ), false ),
			__( 'Check this to use Linear-provided Leadoo robot, if available.', 'linear' )
		);
	}
	*/

	/**
	 * Display terms_of_service field.
	 */
	public function terms_of_service_callback() {
		printf(
			'<input type="checkbox" id="terms_of_service" name="linear_settings[terms_of_service]" required value="1" %s /> %s <a target="_blank" href="%s">%s<a>.',
			checked( '1', self::$linear->get_option( 'terms_of_service' ), false ),
			__( 'Before using Linear service you need to accept our', 'linear' ),
			esc_url( 'https://linear.fi/wp-content/uploads/2022/07/Linear-Yleiset-Toimitusehdot.pdf' ),
			__( 'terms of service', 'linear' )
		);
	}

	/**
	 * Display shortcode usage
	 */
	public function shortcode_guide( $anchor_id = '' ) {
		echo('
			<div class="linear__advanced__section" id="' . $anchor_id . '">
				<h2>' . __( 'Shortcodes', 'linear' ) . '</h2>
				<div>' .
				__('You can add listings according to your own liking wherever you want on the site with the help of shortcodes.', 'linear') . 
				' ' . __('This is handy for sites which are not using the Gutenberg editor as the listing is provided primarily as a Block, but can be also utilized with Shortcodes.', 'linear') . 
				' ' . __('The shortcodes support some additional settings which affect how the listing is displayed.', 'linear') . '</div>
				<br>
				
				<div class="linear__advanced__shortcode__section">
					<p><strong>' . __('Simple example:', 'linear') . '</strong></p>
					<pre><code class="language-html">
[linear_block_listings]
					</code></pre>
				</div>

				<div class="linear__advanced__shortcode__section">
					<p><strong>' . __('Advanced example:', 'linear') . '</strong></p>
					<pre><code class="language-html">
[linear_block_listings type="apartments" per_page="4" loadmore="false" classes="text-color-brand" filters="false" range_sliders="false" order_by="true"]
					</code></pre>
					<br><br>
					<p><strong>' . __('Supported arguments:', 'linear') . '</strong></p>
					<ol>
						<li>
							<p><strong>' . __('Type', 'linear') . ' (type)</strong></p>
							<p>' . __('The type indicates if we want to show apartment, rentals or business premises. Default is "all"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"all", "apartments", "rent_apartments", "business_premises"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings type="all"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Per page', 'linear') . ' (per_page)</strong></p>
							<p>' . __('The per page indicates how many listings we want to show, and how many more are shown upon "Load more"-button click. Default is "8"', 'linear') . '</p>
							<p>' . __('Accpeting positive numeric values:', 'linear') . ' <code>' . __('*Any positive number from 1-999*', 'linear') . '</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings per_page="8"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Load more', 'linear') . ' (loadmore)</strong></p>
							<p>' . __('The options to enable/disable the "Load more"-button. Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings loadmore="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Classes', 'linear') . ' (classes)</strong></p>
							<p>' . __('Add one or several classes to add to the block. Default is ""', 'linear') . '</p>
							<p>' . __('Accepting string value:', 'linear') . ' ' . __('eg.', 'linear') . ' <code>"class_name_one class_name_two"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings classes="my_extra_class my_other_extra_class"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Filters', 'linear') . ' (filters)</strong></p>
							<p>' . __('You can chose if you want to show or hide the filters for the listing. Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings filters="true"]
								</code></pre>
							</p>
						</li>

						<li>
							<p><strong>' . __('Range sliders', 'linear') . ' (range_sliders)</strong></p>
							<p>' . __('Some filterings support range sliders for eg. price or rent. You can decide if you want to show these or not. Requires that filters are used! Default is "true"', 'linear') . '</p>
							<p>' . __('Acceptable values:', 'linear') . ' <code>"true", "false"</code></p>
							<p>
								<span> ' . __('eg.', 'linear') . ' </span>
								<pre><code class="language-html">
[linear_block_listings range_sliders="true"]
								</code></pre>
							</p>
						</li>
					</ol>
				</div>
			</div>
		');
	}

	/**
	 * Display rest API guide
	 */
	public function rest_api_guide( $anchor_id = '' ) {
		$rest_endpoint = get_home_url() . '/wp-json/linear/' . $this->api_version . '/';

		echo('
			<div class="linear__advanced__section" id="' . $anchor_id . '">
				<h2>REST-API usage</h2>
				<div>' . __('The Linear plugin exposes a few rest endpoints which allow you to fetch listings. The plugin handles everything with fetching the data and saving it in a transient so that you don\'t have to worry about that.', 'linear') . '</div>
				
				<div>
					<p><strong>' . __('Endpoints:', 'linear') . ' <a href="' . $rest_endpoint . '">' . $rest_endpoint . '</a></strong></p>
					<pre><code class="language-json">
{
    "namespace": "linear/v1",
    "routes": {
        "/linear/v1": {},
        "/linear/v1/listings/all": {},
        "/linear/v1/listings/apartments": {},
        "/linear/v1/listings/rentals": {},
        "/linear/v1/listings/business_premises": {},
        "/linear/v1/listings/SINGLE_LISTING_ID": {}
    }
}
					</code></pre>
				</div>

				<div>' . __('If you want you can also access the same data via WordPress filters.', 'linear') . '</div>
				
				<div>
					<p><strong>' . __('Available filters:', 'linear') . '</strong></p>
					<pre><code class="language-php">
apply_filters( "linear_listings_all", null )
apply_filters( "linear_listings_apartments", null )
apply_filters( "linear_listings_rentals", null )
apply_filters( "linear_listings_business_premises", null )
apply_filters( "linear_listing", "SINGLE_LISTING_ID" )
					</code></pre>
				</div>

				<div>
					<p>' . __( 'And you can use them straight like this', 'linear' ) . ':</p>
					<pre><code class="language-php">
$listings = apply_filters( "linear_listings_all", null );

foreach( $listings as $listing ){
    echo $listing[\'address\'];
}
					</code></pre>
				</div>
			</div>
		');
	}
}
