<?php
/**
 * Linear content template functionality
 *
 * @package Linear
 */

/**
 * Controls the use of custom templates for linear plugin content.
 */
class Linear_Templater {

	/**
	 * Core plugin class instance.
	 *
	 * @var Linear
	 */
	protected static $linear;

	/**
	 * Linear pages that are using linear templates.
	 *
	 * @var array
	 */
	protected static $linear_pages;
	protected static $api_version;

	/**
	 * Defines class properties.
	 */
	public function __construct() {
		if ( is_null( self::$linear ) ) {
			self::$linear = Linear::get_instance();
		}

		self::$linear_pages = array(
			'listings_page'  => self::$linear->get_option( 'listings_page' ),
			'rentals_page'   => self::$linear->get_option( 'rentals_page' ),
			'workplace_page' => self::$linear->get_option( 'workplace_page' )
		);

		self::$api_version = ( self::$linear->get_option( 'api_key' ) ? 'v2' : 'v1' );
	}

	/**
	 * Return special template for list and single listing pages.
	 * 
	 * @return string
	 */
	public function include_templates( $template ) {
		global $post;

		if ( $post && in_array( $post->ID, array_values( self::$linear_pages ) ) ) {
			global $wp_query;

			$id = $wp_query->get( 'linear_listing_id', false );
			if ( $id === false && isset( $_GET['linear_listing_id'] ) ) {
				$id = sanitize_key( $_GET['linear_listing_id'] );
			}

			// If single listing or listings view
			if ( $id !== false ) {
				$listing_data = null;
				if ( Linear::validate_listing_id( $id ) ) {
					$linear_middleware = self::$linear->get_middleware();
					$listing_data = $linear_middleware->get_single( $id );
				}

				// Check if listing exists, if not redirect to 404
				if ( empty( $listing_data ) ) {
					global $wp_query;
					$wp_query->set_404();
					status_header( 404 );
					return Linear::get_template_path( 'linear-404.php' );
				}

				// Set listing data to global query
				set_query_var('linear_listing', $listing_data );

				/**
				 * Commons things between API-versions before handling separately edge cases
				 */

				// SEO optimisation
				$title = trim( $listing_data['address'] . ( isset( $listing_data['gate'] ) ? ' ' . $listing_data['gate'] : '' ), ' ');

				// SOME-preview workaround
				$description = $title;
				$image_url = '';

				/**
				 * Handle certain data in different ways depending on which API version is used
				 */

				$price = ( isset( $listing_data['formatted_debtFreePrice'] ) ? $listing_data['formatted_debtFreePrice'] : ( isset( $listing_data['formatted_rent'] ) ? $listing_data['formatted_rent'] : '') );

				// SEO optimisation
				$title = implode(
					', ', 
					array_filter(
						array( 
							isset( $title ) ? $title : false,
							isset( $listing_data['districtFree'] ) ? $listing_data['districtFree'] : false,
							isset( $listing_data['city'] ) ? $listing_data['city'] : false,
							isset( $price ) ? $price : false
						)
					)
				);

				// Add clients sitename
				$title = $title . ' | ' . get_bloginfo( 'name' );

				// Set default template
				$template = 'linear-listing.php';

				/**
				 * Further common parts
				 */

				// "$title_parts" contains " - linear-plugin" which might not be favored by some clients.
				add_filter(
					'document_title_parts',
					function ( $title_parts ) use ( $title ){
						if ( ! empty( $title ) ) {
							$title_parts['title'] = $title;
						}
						return $title_parts;
					}
				);

				// hacky bugfix: listing page always has no canonical URL (WP generated it wrong, which caused issues with facebook preview links)
				add_filter( 'get_canonical_url', function( $canonical_url, $post ) {
					global $wp;
					return get_home_url() . '/' . $wp->request;
				}, 10000, 200 );

				// SOME-preview workaround
				if ( isset( $listing_data['districtFree'] ) && isset( $listing_data['typeOfApartment'] ) ) {
					$description .= ($listing_data['typeOfApartment'] ? ' - ' . $listing_data['typeOfApartment'] : '');
				}
				if ( isset( $listing_data['thumbnails'] ) && !empty( $listing_data['thumbnails'] ) ) {
					$image_url = $listing_data['thumbnails'][0];
				} else if( isset( $listing_data['images'] ) && !empty( $listing_data['images'] ) ){
					$image_url = $listing_data['images'][0]->compressed;
				}
				add_action(
					'wp_head',
					function () use ( $description, $image_url ) {
						if ( ! empty( $description ) ) {
							echo '<meta name="description" content="' . $description . '">';
						}
						if ( ! empty( $image_url ) ) {
							echo '<meta property="og:image" content="' . esc_url( $image_url ) . '">';
						}
					},
					50
				);

				// Yoast SEO exceptions
				add_filter( 'wpseo_title', function( $yoast_title ) use ( $title ){
					return $title;
				});
				add_filter( 'wpseo_opengraph_title', function( $yoast_title ) use ( $title ){
					return $title;
				});
				add_filter( 'wpseo_canonical', function( $canonical ) {
					global $wp;
					return get_home_url() . '/' . $wp->request;
				}, 20 );
				add_filter( 'wpseo_opengraph_url', function( $canonical ) {
					global $wp;
					return get_home_url() . '/' . $wp->request;
				}, 20 );
				add_filter( 'wpseo_twitter_image', function( $image ) use ( $image_url ){
					return esc_url( $image_url );
				}, 20 );

				// The SEO Framework exceptions
				add_filter( 'the_seo_framework_title_from_custom_field', function( $tsf_title, $args ) use ( $title ) {
					return $title;
				}, 10, 2 );

				// All in one SEO Framework exception
				add_filter( 'aioseo_title', function( $aioseo_title ) use ( $title ) {
					return $title;
				});

				 // Rank Math exception
				 add_filter( 'rank_math/frontend/title', function( $rank_math_title ) use ( $title ) {
					return $title;
				});

				// Seopress exception
				add_filter('seopress_titles_title', function ( $html ) use ( $title ) { 
					return $title;
				});

				// Squirrly SEO exception
				add_filter('sq_title', function ( $sq_title ) use ( $title ) {
					return $title;
				});

				return Linear::get_template_path( $template );

			}

			// Handle special rule for current page
			$options = get_option( 'linear_settings' );
			$linear_page = isset( $options['listings_page'] ) ? $options['listings_page'] : -1;
			$listings_page = get_post( $linear_page );
			if( $listings_page->ID === $post->ID ) {
				// Check if default page template is in use
				$listings_page_template = get_page_template_slug( $listings_page );

				if( $listings_page_template === "" ){
					$template = 'plain-listings.php';
					return Linear::get_template_path( $template );
				}
			}
		}
		return $template;
	}

	/**
	 * Add and enqueue necessary style for special templates.
	 */
	public function enqueue_style() {
		global $post;

	}

	public function enqueue_block_editor_assets() {
		wp_enqueue_script(
			'linear-editor', 
			LINEAR_PLUGIN_URL . get_asset( 'editor.js', 'admin' ),
			[ 'wp-blocks', 'wp-dom-ready', 'wp-edit-post' ],
			LINEAR_VERSION
		);
	}

	public function enqueue_editor_styles() {
		add_editor_style( get_asset( 'editor.css', 'admin' ) );
	}

	public function enqueue_frontend_scripts() {
		global $post;

		if ( $post && in_array( $post->ID, array_values( self::$linear_pages ) ) ) {

			// wp_deregister_script('jquery');
			// wp_enqueue_script('jquery', '/wp-includes/js/jquery/jquery.min.js', [], null, false);		

			wp_enqueue_script(
				'linear-frontend',
				LINEAR_PLUGIN_URL . get_asset( 'frontend.js' ),
				[ 'wp-polyfill' ],
				LINEAR_VERSION,
				true
			);
		
			wp_enqueue_style(
				'linear-frontend',
				LINEAR_PLUGIN_URL . get_asset( 'frontend.css' ),
				[],
				LINEAR_VERSION
			);

			/*
			if( !get_query_var('linear_listing') ){
				return;
			}
			*/

			// Legacy color mappings
			$primary_color = self::$linear->get_option( 'primary_color', '#1890ff' );
			$solid_color   = self::$linear->get_option( 'solid_color', '#096dd9' );
			$outline_color = self::$linear->get_option( 'outline_color', '#0050b3' );

			wp_add_inline_style(
				'linear-frontend',

				"
				#linear .linear-c-button--primary.linear-c-button--solid,
				#linear .linear-c-tag {
					background-color: $primary_color;
				}

				#linear .linear-c-button--primary.linear-c-button--solid,
				#linear .linear-c-button--primary.linear-c-button--outline {
					border-color: $primary_color;
				}

				#linear .linear-c-button--primary.linear-c-button--outline,
				#linear .linear-c-gallery-slider__fullscreen-toggle,
				#linear .linear-c-gallery-slider__thumbnails .linear-c-gallery-slider__slide--360-view .linear-c-gallery-slider__content,
				#linear a,
				#linear .linear-c-gallery-slider__arrow,
				#linear .mapboxgl-marker {
					color: $primary_color;
				}

				#linear .linear-u-text-primary {
					color: $primary_color !important;
				}

				#linear .linear-c-button--primary.linear-c-button--solid:hover,
				#linear .linear-c-button--primary.linear-c-button--solid:focus,
				#linear .linear-c-button--primary.linear-c-button--solid:active {
					background-color: $solid_color;
					border-color: $solid_color;
				}

				#linear .linear-c-button--primary.linear-c-button--outline:hover,
				#linear .linear-c-button--primary.linear-c-button--outline:focus,
				#linear .linear-c-button--primary.linear-c-button--outline:activer {
					color: $outline_color;
				}

				#linear .linear-c-button--primary.linear-c-button--outline:hover,
				#linear .linear-c-button--primary.linear-c-button--outline:focus,
				#linear .linear-c-button--primary.linear-c-button--outline:active {
					border-color: $outline_color;
				}

				#linear .linear-c-card__icon,
				#linear .linear-c-icon-info__icon,
				#linear .linear-c-icon-info__icon svg,
				#linear .linear-c-actions__icon,
				#linear .linear-icon-color-theme,
				#linear .linear-icon-color-theme svg,
				#linear .linear-icon-color-plugin,
				#linear .linear-icon-color-plugin svg,
				#linear .linear-c-card__collapse-toggle:after {
					color: " . Linear\Templates\transform_hex_to_rgba( $primary_color, "1.0" ) . ";
				}
				"
			);

		}		
	}

	public function enqueue_admin_scripts() {
		wp_enqueue_style( 'wp-color-picker' );
		
		if ( ! did_action( 'wp_enqueue_media' ) ) {
			wp_enqueue_media();
		}

		wp_enqueue_script(
			'linear-admin',
			LINEAR_PLUGIN_URL . get_asset( 'admin.js', 'admin' ),
			[ 'jquery', 'wp-color-picker' ],
			LINEAR_VERSION,
			true
		);

		wp_enqueue_style(
			'linear-admin',
			LINEAR_PLUGIN_URL . get_asset( 'admin.css', 'admin' ),
			[],
			LINEAR_VERSION
		);
	}

	/**
	 * Enqueue necessary script for special template.
	 */
	public function enqueue_script() {
		global $post;
		if ( $post && in_array( $post->ID, array_values( self::$linear_pages ) ) ) {
			// wp_enqueue_script( 'linear_main', plugins_url( 'assets_old/js/main.js', __DIR__ ), array( 'jquery' ), '1.0.0', true);
			wp_localize_script(
				'linear_main',
				'linear_data',
				array( 
					'ajax_url'          => admin_url( 'admin-ajax.php' ),
					'loading'           => __( 'Loading more', 'linear' )
				)
			);
		}
	}

	public function include_listings_content( $content ) {
		global $post, $wp_query;

		if ( $post && in_array( $post->ID, array_values( self::$linear_pages ) ) ) {
			$id = $wp_query->get( 'linear_listing_id', false );
			if ( $id === false && isset( $_GET['linear_listing_id'] ) ) {
				$id = sanitize_key( $_GET['linear_listing_id'] );
			}

			if ( $id === false ) {

				$defaults = [
					'type' => 'all',
					'per_page' => '8',
					'filters' => 'true',
					'loadmore' => 'true',
					'classes' => '',
					'range_sliders' => 'true',
					'order_by' => 'true',
				];

				if ( self::$linear_pages['rentals_page'] == $post->ID ) {

					$defaults = array_merge( $defaults, [
						'type' => 'rent_apartments',
					]);

				} elseif ( self::$linear_pages['workplace_page'] == $post->ID ) {

					$defaults = array_merge( $defaults, [
						'type' => 'business_premises',
					]);

				} else {

					$defaults = array_merge( $defaults, [
						'type' => 'apartments',
					]);

				}

				ob_start();
				
					echo do_shortcode('[linear_block_listings 
						type="' . $defaults['type'] . '" 
						per_page="' . $defaults['per_page'] . '" 
						loadmore="' . $defaults['loadmore'] . '" 
						classes="' . $defaults['classes'] . '" 
						filters="' . $defaults['filters'] . '"
						range_sliders="' . $defaults['range_sliders'] . '"
						order_by="' . $defaults['order_by'] . '"
					]');

				$content =  $content . ob_get_clean();
			}
		}
	
		return $content;
	}

	public function maybe_use_fonts_frontend( $classes ){
		// Add additional class to body to declare if CSS should use plugin fonts
		if ( !self::$linear->get_option( 'theme_fonts' ) ) {
			$classes[] = 'use-linear-fonts';
		}

		return $classes;
	}

	public function maybe_use_fonts_admin( $classes ){
		if ( !self::$linear->get_option( 'theme_fonts' ) ) {
			$classes .= ' use-linear-fonts';
		}

		return $classes;
	}

	// Load fonts if settings demand it
	public function enqueue_global_styles(){
		global $post;

		if ( !self::$linear->get_option( 'theme_fonts' ) ) {
			wp_enqueue_style( 'linear-fonts', 'https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600;700&display=swap' );
		}
	}

	// Linear::get_template_path( plain-listings.php );
	public function template_creation( $page_template ){
		if ( get_page_template_slug() == 'plain-listings.php' ) {
			$page_template = Linear::get_template_path( 'plain-listings.php' );
		}
		return $page_template;
	}

	public function template_population( $post_templates, $wp_theme, $post, $post_type ){
		$post_templates['plain-listings.php'] = __('Linear listings');

		return $post_templates;
	}
}